//+------------------------------------------------------------------+
//|                                            FTF_MovingAverage.mq4 |
//|                                      Copyright Herb Spirit, Inc. |
//|                                    http://www.herbspirit.com/mql |
//+------------------------------------------------------------------+
#property copyright "Copyright Herb Spirit, Inc."
#property link      "http://www.herbspirit.com/mql"

#define TF_SAME	0
#define TF_SMALL	1
#define TF_LARGE	2

//---- indicator settings
#property  indicator_chart_window
#property  indicator_buffers 2
#property  indicator_color1  Red

//---- indicator parameters
extern int     maTF;
extern int  	maPeriod=8;
extern int     maMethod=MODE_EMA;
extern int     maPrice=PRICE_CLOSE;
extern int		maShift=0;
//---- buffers
double mainbuf[];
double extrabuf[];
//---- global vars
int tf,maperiod,mamethod,maprice,mashift;
bool expertvalid;
int tfmode;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- validate parms
	expertvalid=ValidateParms();
	if(!expertvalid)
		return(0);	
//---- set indicator style
	SetIndexBuffer(0,mainbuf);
	SetIndexBuffer(1,extrabuf);
	switch(tfmode)
	{
		case TF_SAME:
		case TF_LARGE:
		{
			SetIndexStyle(0,DRAW_LINE);
			SetIndexStyle(1,DRAW_NONE);
			break;
		}
		case TF_SMALL:
		{
			SetIndexStyle(0,DRAW_HISTOGRAM);
			SetIndexStyle(1,DRAW_HISTOGRAM);
			break;
		}
	}
//----
	return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
	return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
	if(!expertvalid)
		return(0);
	int limit=MathMin(Bars-IndicatorCounted(),Bars-maperiod);
	for(int i=0;i<limit;i++)
	{
		switch(tfmode)
		{
			case TF_SAME:
				mainbuf[i]=iMA(NULL,tf,maperiod,mashift,mamethod,maprice,i);
				break;
			case TF_SMALL:
				SetSmallTF(i);
				break;
			case TF_LARGE:
				SetLargeTF(i);
				break;
		}
	}
	return(0);
}
//+------------------------------------------------------------------+
//| set small timeframe
//+------------------------------------------------------------------+
void SetSmallTF(int shift)
{
	int smallshift=iBarShift(NULL,tf,Time[shift]);
	datetime cuttime=Time[shift]+Period()*60-1;
	double highma=0;
	double lowma=9999999;
	int check=0;
	while(iTime(NULL,tf,smallshift)<cuttime&&smallshift>=0)
	{
		double ma=iMA(NULL,tf,maperiod,mashift,mamethod,maprice,smallshift);
		if(ma>highma)
			highma=ma;
		if(ma<lowma)
			lowma=ma;
		smallshift--;
		check++;
		if(check>1000)
		{
			Print("infinite loop in SetSmallTF");
			break;
		}
	}
	mainbuf[shift]=highma;
	extrabuf[shift]=lowma;
}
//+------------------------------------------------------------------+
//| set small timeframe
//+------------------------------------------------------------------+
void SetLargeTF(int shift)
{
	int largeshift=iBarShift(NULL,tf,Time[shift]);
	int check =0;
	while(iTime(NULL,tf,largeshift)>Time[shift])
	{
		largeshift++;
		check++;
		if(check>1000)
		{
			Print("infinite loop in SetLargeTF");
			break;
		}
	}
	int startshift=iBarShift(NULL,0,iTime(NULL,tf,largeshift));
	double value=iMA(NULL,tf,maperiod,mashift,mamethod,maprice,largeshift);
	while(startshift>=shift)
	{
		mainbuf[startshift]=value;
		startshift--;
	}
}
//+------------------------------------------------------------------+
//| Validate input parms
//+------------------------------------------------------------------+
bool ValidateParms()
{
//---- validate timeframe
//---- do your validation against valid timeframes
	if(tf<0)
	{
		Print("Invalid timeframe ",maTF);
		return(false);
	}
	if(tf==0)
	{
		tf=Period();
	}
//---- set indicator timeframe relation to current timeframe
	if(tf<Period())
		tfmode=TF_SMALL;
	else
		if(tf>Period())
			tfmode=TF_LARGE;
		else
			tfmode=TF_SAME;
//---- validate ma period
	if(maPeriod<1)
	{
		Print("Invalid period ",maPeriod);
		return(false);
	}
	else
		maperiod=maPeriod;
//---- ma method
//---- do your validation of valid ma methods 
	mamethod=maMethod;
	if(mamethod<0)
	{
		Print("Invalid Method constant",maMethod);
		return(false);
	}
//---- ma price
//---- do your validation of price constant 
	maprice=maPrice;
	if(maprice<0)
	{
		Print("Invalid Price constant ",maPrice);
		return(false);
	}
	mashift=maShift;
	return(true);
}